#!/bin/bash

### The simulation part of the neural network. This is run by individual chunks in parallel with each other

### 1. Creates directories for each member if they havent already done so, i.e. in the first generation
### 2. Edits the namelists of each member using the corresponding row of the 00.csv parameters file
### 3. Runs the simulation
### 4. Uses the outputs, reference bias and observed data to collect the cost and save as a .txt

## ALL SHOULD BE THE SAME AS THOSE IN pre.sh
SCRHOME=/gpfs/scratch/bsc32/bsc32786/
MODEL=${SCRHOME}%EXPID%/EXP00/
OBS=${SCRHOME}pisces1d/ORCA2_OFF_PISCES/obs/Mprof_6901486_L31_month_2015-01-01_2015-12-31.nc
MODELFILE=PISCES_1m_19600101_19691231_ptrc_T.nc


### 1. Creates directories for each member if they havent already done so, i.e. in the first generation

shopt -s nullglob
numfiles=(${MODEL}parameters/*)
numfiles=${#numfiles[@]}

INPUTS=input/
XIOS=xios-2.5/
NEMO=release-4.0/

chunk=%CHUNK%
printf -v chunk "%04d" $chunk

if [ $numfiles -eq $((1)) ]; then
    # MAKE EXPID DIRECTORY AND COPY ALL THE CONTENT FROM EXP00 (.XML, EXECUTABLES, AND NAMELISTS)
    cd ${MODEL}
    mkdir -p $chunk
    cd $chunk
    cp -R ${MODEL}default/nemo .
    cp -R ${MODEL}default/namelist* .
    cp -R ${MODEL}default/*.xml .
    cp -R ${MODEL}cropped/* .
fi


### 2. Edits the namelists of each member using the corresponding row of the 0000.csv parameters file

VARIABLES=variables.csv
NAMELIST=namelist_pisces_ref

cd ${MODEL}$chunk/

module load python

function changenls {

nls="$(python - $1 $2 $3 $4 << END
# inputs:
# 1. Parameters file (0000.csv)
# 2. Variables file (variables.csv)
# 3. Namelist (namelist_pisces_ref)
# 4. Chunk number

import os
import pandas as pd
import numpy as np

values=np.genfromtxt('$1')
names=pd.read_csv('$2')

chunk=int('$4')-1

with open('$3','r') as nl:
    with open('temp','w+') as t:
        lines=nl.readlines()
        for l in lines:
            for j in range(len(values[chunk])):
                if names['var'][j]+' ' in l and "=" in l:
                    l = '   '+ names['var'][j] + '  =  ' + str(values[chunk][j]) + '\n'
            t.write(l)

os.remove('$3')
os.rename('temp','$3')

END
)"
}

changenls ${MODEL}parameters/0000.csv ${MODEL}$VARIABLES ${MODEL}$chunk/$NAMELIST %CHUNK%


### 3. Runs the simulation

cd ${MODEL}$chunk

module load intel/2017.4
module load impi/2017.4
module load netcdf/4.2
module load hdf5/1.8.19
module load perl

mpirun -np 1 ./nemo

### 4. Uses the outputs, reference bias and observed data to collect the cost and save as a .txt

module load netcdf/4.4.1.1

function getcost {

cost="$(python - $1 $2 $3 $4 << END
# $1 - path to reference bias, $2 - path to observed data $3 - Path to output

from netCDF4 import Dataset
import numpy as np
import os
import math

# get the reference bias

os.chdir('$1')
bbp700refbias=np.genfromtxt('bbp700refbias.csv')
bPOCrefbias=np.genfromtxt('bPOCrefbias.csv')
chlarefbias=np.genfromtxt('chlarefbias.csv')

# get the observed data

def getobsdata(ds,varname):
    A = np.array(obs.variables[varname])
    A[A == 0.0] = np.nan
    A=np.log10(A)
    return A

obs=Dataset('$2')
bbp700obs = getobsdata(obs,'bbp700_despiked')
bPOCobs = getobsdata(obs,'bbp700_spike')
chlaobs = getobsdata(obs,'chla_cal')
depths=np.array(obs.variables['deptht'])
obs.close()

# get the model data
os.chdir('$3')
mod = Dataset('$4')
bbp700mod = (np.array(mod.variables['POC'])[-60:, :, 1, 1]+np.array(mod.variables['PHY'])[-60:, :, 1, 1])*0.001
bPOCmod = (np.array(mod.variables['GOC'])[-60:, :, 1, 1]+np.array(mod.variables['PHY2'])[-60:, :, 1, 1])*0.001
chlamod = (np.array(mod.variables['DCHL'])[-60:, :, 1, 1]+np.array(mod.variables['NCHL'])[-60:, :, 1, 1])
mod.close()


def editmoddata(arr,d):
    arr=arr[:,:np.shape(d)[0]]
    for i in range(12):
        arr[i] = np.mean(arr[i::12],0)
    arr = arr[:12]
    arr[arr == 200000039192547557376] = np.nan
    arr=arr[:,:np.shape(arr)[1]]
    arr=np.log10(arr)
    return arr

bbp700mod=editmoddata(bbp700mod,depths)
bPOCmod=editmoddata(bPOCmod,depths)
chlamod=editmoddata(chlamod,depths)

#def gradperiod(A,d):
#    A=np.concatenate(([A[-1]],A,[A[0]]),axis=0)
#    B=np.gradient(A,1,d)
#    return B[0][1:-1]#+B[0][1:-1]
#
#bbp700obs=gradperiod(bbp700obs,depths)[:,10:]
#bbp700mod=gradperiod(bbp700mod,depths)[:,10:]
#chlaobs=gradperiod(chlaobs,depths)[:,:17]
#chlamod=gradperiod(chlamod,depths)[:,:17]

bbp700obs=bbp700obs[:,10:]
bbp700mod=bbp700mod[:,10:]
bPOCobs=bPOCobs[:,10:]
bPOCmod=bPOCmod[:,10:]
chlaobs=chlaobs[:,:17]
chlamod=chlamod[:,:17]

# standardising the data
bbp700std=np.nanstd(bbp700obs)
bbp700mn=np.nanstd(bbp700obs)
bbp700obs=(bbp700obs-bbp700mn)/bbp700std

bPOCstd=np.nanstd(bPOCobs)
bPOCmn=np.nanstd(bPOCobs)
bPOCobs=(bPOCobs-bPOCmn)/bPOCstd

chlastd=np.nanstd(chlaobs)
chlamn=np.nanstd(chlaobs)
chlaobs=(chlaobs-chlamn)/chlastd

# calculating the statistical parameters

names=['bias','normstd','R','RMSE','cost']

bias=np.nanmean(bbp700mod-bbp700obs)
mbias=bias/bbp700refbias
mstd=np.nanstd(bbp700mod)/np.nanstd(bbp700obs)
R=float(np.ma.corrcoef(np.ma.masked_invalid(bbp700obs.flatten()), np.ma.masked_invalid(bbp700mod.flatten()))[0,1])
S3=1.-(((1.+R)**4.)/(4.*(mstd**2+mstd**(-2)+2))) #S2
RMSE=math.sqrt(np.nansum((bbp700obs-bbp700mod)**2)/bbp700obs.size)
ST=math.sqrt((mbias**2)+(S3**2))

stats=[bias,mstd,R,RMSE,ST]
for i in range(len(names)):
    np.savetxt('bbp700'+names[i]+'.csv',[stats[i]])

bias=np.nanmean(bPOCmod-bPOCobs)
mbias=bias/bPOCrefbias
mstd=np.nanstd(bPOCmod)/np.nanstd(bPOCobs)
R=float(np.ma.corrcoef(np.ma.masked_invalid(bPOCobs.flatten()), np.ma.masked_invalid(bPOCmod.flatten()))[0,1])
S3=1.-(((1.+R)**4.)/(4.*(mstd**2+mstd**(-2)+2))) #S2
RMSE=math.sqrt(np.nansum((bPOCobs-bPOCmod)**2)/bPOCobs.size)
ST=math.sqrt((mbias**2)+(S3**2))

stats=[bias,mstd,R,RMSE,ST]
for i in range(len(names)):
    np.savetxt('bPOC'+names[i]+'.csv',[stats[i]])

bias=np.nanmean(chlamod-chlaobs)
mbias=bias/chlarefbias
mstd=np.nanstd(chlamod)/np.nanstd(chlaobs)
R=float(np.ma.corrcoef(np.ma.masked_invalid(chlaobs.flatten()), np.ma.masked_invalid(chlamod.flatten()))[0,1])
S3=1.-(math.exp(-((mstd-1.)**2)/0.18))*((1+R)/2)
RMSE=math.sqrt(np.nansum((chlaobs-chlamod)**2)/chlaobs.size)
ST=math.sqrt((mbias**2)+(S3**2))

stats=[bias,mstd,R,RMSE,ST]
for i in range(len(names)):
    np.savetxt('chla'+names[i]+'.csv',[stats[i]])

END
)"
}

getcost ${MODEL}default/ $OBS $MODEL${chunk} $MODELFILE
